#include <ESP8266WiFi.h>
#include <ESP8266WiFiMulti.h>

ESP8266WiFiMulti WiFiMulti;
const uint16_t port = 80;
// IP of my local ESP8266 server.
const char * host = "192.168.1.6";

// Pin declarations, two transmit pins (IR LEDs) and two receive pins
const int transmitPinOne = 4; 
const int transmitPinTwo = 13; 
const int receivePinOne = 16; 
const int receivePinTwo = 12; 
// On board blue stat LED
const int statLED = 5; 

int irResp = 0; // Which receiver broke?
int led = 1; // Which LED are we transmitting on?
int numChecks = 50; // Number of pulses per burst
int personCount = 0; // Number of people in the room
long int currentTime;

bool lightsOn = false; 


void setup()
{

  Serial.begin(115200);

  // Connecting to WiFi....
  WiFi.mode(WIFI_STA);
  WiFiMulti.addAP("Sparkle Mausoleum", "saile123"); 
  WiFiClient client;  // Remove

  while (WiFiMulti.run() != WL_CONNECTED) {
    Serial.print(".");
    delay(500);
  }

  Serial.println("");
  Serial.println("WiFi connected");
  Serial.println("IP address: ");
  Serial.println(WiFi.localIP());

 // Serial.print("Connecting to Relay Server...");
 // Serial.println(host);
 
  // Remove vv
  while(!client.connect(host, port)) {
    Serial.println("."); 
    delay(500); 
  }
  Serial.println("Connected!"); 
  client.println("/relay/1");
  Serial.println("Read from server: "); 
  String line = client.readStringUntil('\r'); 
  Serial.println(line);
  // Remove ^^

  pinMode(transmitPinOne, OUTPUT); 
  pinMode(transmitPinTwo, OUTPUT); 

  // Receivers are active LOW
  pinMode(receivePinOne, INPUT);
  pinMode(receivePinTwo, INPUT);
  digitalWrite(receivePinOne, HIGH); 
  digitalWrite(receivePinTwo, HIGH); 
  pinMode(statLED, OUTPUT);
  
}
void loop()
{
  currentTime = millis(); // Keep time up to date. 
  digitalWrite(statLED, HIGH); //Turn OFF the LED

  // If a receiver is tripped then we'll get which one was broken here. I'm
  // giving the function the led to check, first one and then two. 
  irResp = IR_stream(led); 

  // Check the results of IR_stream here. If the first is broken then someone may be entering
  // the room and so we're setting up a fixed amount of time for a break
  // on the second IR stream.
  if(irResp == 1) {
    currentTime = millis();  
    Serial.println("One"); 
    led++;
    while( millis() - currentTime < 2000){
      if(IR_stream(led) == 2){
        lightsOn = true; 
        personCount++; // Party Peoples in the room.
        Serial.print("Person Count: "); 
        Serial.println(personCount);
        digitalWrite(statLED, LOW); 
        sendRelayOn();
        delay(500); //debounce of sorts
        break;
      }
    }
  }
  // Likewise if the second IR_stream was broken that may mean someone is
  // leaving the room. We're setting up a fixed time for that person to leave
  // by looking for a break on the first IR stream.
  else if(irResp == 2 && lightsOn == true) { 
    currentTime = millis();  
    led--;
    while( millis() - currentTime < 2000){
      if(IR_stream(led) == 1){
        if (personCount > 0)
          personCount--; //Second person is gone. 
        if (personCount == 0){
          sendRelayOff();
          lightsOn = false; 
        }  
        Serial.print("Person Count: "); 
        Serial.println(personCount);
        digitalWrite(statLED, LOW); 
        delay(500); //debounce of sorts
      }
    }
  }

  delay(1);

  // We're switching between led one and two. 
  if (led == 2)
    led = 1;  
  else 
    led++;
}

// This function handles the burst of IR to the receiver. It sends up 50 pulses
// to the IR receiver befor reading its' state. All together the
// time totals ~1.1 ms before reading the receiver.
int IR_stream(int LED){

  for (int i = 0; i < numChecks; i ++) {

    switch(LED) {
      case 1:
        digitalWrite(transmitPinOne, HIGH); 
        break;
      case 2:
        digitalWrite(transmitPinTwo, HIGH); 
        break;
      
    }

    delayMicroseconds(6); 

    switch(LED) {
      case 1:
        digitalWrite(transmitPinOne, LOW); 
        break;
      case 2:
        digitalWrite(transmitPinTwo, LOW); 
        break;
    }

    delayMicroseconds(17); 
  }

  if(LED == 1) {
    if(digitalRead(receivePinOne) == HIGH) 
      return(1); 
  }

  if(LED == 2) {
    if(digitalRead(receivePinTwo) == HIGH) 
      return(2); 
  }

  return(0);    

}

// These two functions handle sending the on and off command to the ESP8266
// server.
int sendRelayOn(){

    WiFiClient client; 
    if(!client.connect(host, port)) 
      Serial.println("Could not connect!"); 
    client.println("/relay/1");
    client.stop();
} 

int sendRelayOff(){

    WiFiClient client; 
    if(!client.connect(host, port)) 
      Serial.println("Could not connect!"); 
    client.println("/relay/2");
    client.stop();
}
