#include <ESP8266WiFi.h>
#include <Wire.h>

// Qwiic Quad Relay commands....
#define RELAY_ADDR 0x6D
#define TOGGLE_RELAY_ONE 0x01
#define RELAY_ONE_STATE 0x05
#define TOGGLE_ALL_OFF 0xA
#define OFF 0x00
#define ON 0x15

const char* ssid = "Sparkle Mausoleum";
const char* password = "saile123";
const uint8_t LED = 4; 

// Create an instance of the server
// specify the port to listen on as an argument
WiFiServer server(80);

void setup() {
  Serial.begin(115200);
  Wire.begin();

  pinMode(LED, OUTPUT);
  digitalWrite(LED, LOW);

  // Connect to WiFi network
  Serial.println();
  Serial.println();
  Serial.print("Connecting to ");
  Serial.println(ssid);

  WiFi.mode(WIFI_STA);
  WiFi.begin(ssid, password);

  // Normally takes 1.5 seconds
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("");
  Serial.println("WiFi connected");

  // Start the server
  server.begin();
  Serial.println("Server started");

  // Print the IP address
  Serial.println(WiFi.localIP());
}

void loop() {
  // Check if a client has connected
  WiFiClient client = server.available();
  if (!client) {
    return;
  }

  // Wait until the client sends some data
  Serial.println("new client");
  while (!client.available()) {
    delay(1);
  }

  // Read the first line of the request
  String req = client.readStringUntil('\r');
  Serial.println(req);
  client.flush();

  int relayOn;
  if (req.indexOf("/relay/1") != -1) {
    relayOn = get_relay_status(); // See if the relay is on...
    if (relayOn == ON) // If the relay is on, then don't toggle it.
      return; 
    Wire.beginTransmission(RELAY_ADDR);
    Wire.write(TOGGLE_RELAY_ONE);
    Wire.endTransmission();
    digitalWrite(LED, HIGH); 
    delay(500); 
    digitalWrite(LED, LOW); 
  }
  else if (req.indexOf("/relay/2") != -1) {
    Wire.beginTransmission(RELAY_ADDR);
    Wire.write(TOGGLE_ALL_OFF); // Turn off all the relays
    Wire.endTransmission();
    digitalWrite(LED, HIGH); 
    delay(500); 
    digitalWrite(LED, LOW); 
  }
  else {
    Serial.println("invalid request");
    client.stop();
    return;
  }

  client.flush();

  // Prepare the response
  String s = "HTTP/1.1 200 OK\r\nContent-Type: text/html\r\n\r\n<!DOCTYPE HTML>\r\n<html>\r\nRelay is ";
  s += (relayOn) ? "high" : "low";
  s += "</html>\n";

  // Send the response to the client
  client.print(s);
  delay(1);
  // The client will actually be disconnected
  // when the function returns and 'client' object is detroyed
}

int get_relay_status() 
{
  Wire.beginTransmission(RELAY_ADDR);
  Wire.write(RELAY_ONE_STATE);
  Wire.endTransmission;
  Wire.requestFrom(RELAY_ADDR, 1); 
  int state = Wire.read(); 
  return state;
}
